
import ij.*;
import ij.gui.*;
import ij.measure.*;
import ij.plugin.*;
import ij.plugin.filter.*;
import ij.plugin.frame.*;
import ij.process.*;
import ij.util.*;
import java.awt.*;
import java.awt.event.*;
import java.lang.Math;
import java.lang.*;
import java.io.FileWriter;
import java.io.*;
import java.util.StringTokenizer;
import java.text.*;

public class ExtractSpectrum implements PlugIn, Measurements {
  private float[] yAxisData;
  private float[] yAxisNormData;
 
  private double startEnergyDbl = 0;
  private double stepEnergyDbl = 0;
  private double endEnergyDbl = 0;
  private double onPeakEnergyDbl = 0;
  private double offPeakEnergyDbl = 0;
  private PlotWindow rawPlot;
  private PlotWindow normPlot;
  private double minThreshold;
  private double maxThreshold;
   
  ImageStack stack;
  ImagePlus imp;
  
  
  public void run(String arg) {}

      
 public PlotWindow extractSpectrum(ImageStack stack2, ImagePlus imp2, Roi roi, double startEnergyDbl2, 
 			double stepEnergyDbl2, double endEnergyDbl2) {
         
	startEnergyDbl = startEnergyDbl2;
	stepEnergyDbl = stepEnergyDbl2;
	endEnergyDbl = endEnergyDbl2;        
	imp = imp2;
	stack = stack2;
 	
 			
        minThreshold = imp.getProcessor().getMinThreshold();
        maxThreshold = imp.getProcessor().getMaxThreshold();
        yAxisData = getZAxisProfile(roi, minThreshold, maxThreshold);

        float[] temp = new float[yAxisData.length]; 
        
        
        
        for (int i = 0; i<yAxisData.length; i++)
        {
        		temp[i] = yAxisData[i];
        
        }
        
        if(imp.getType() == 1)
        {
        		for (int i = 0; i<yAxisData.length; i++)
        		{
        			yAxisData[yAxisData.length-i-1] = temp[i]-32768;
        
        		}
			}
			else
			{
				for (int i = 0; i<yAxisData.length; i++)
        		{
        			yAxisData[yAxisData.length-i-1] = temp[i];
        
        		}
        	}
			
			
			
        if (yAxisData!=null ) {
            float[] x = new float[yAxisData.length];
            for (int i=0; i<x.length; i++)
                x[i] = (float)(endEnergyDbl +(Math.abs(stepEnergyDbl) * i));
            Rectangle r = imp.getRoi().getBoundingRect();
            rawPlot = new PlotWindow(imp.getTitle()+"-"+r.x+"-"+r.y, "Energy", "Mean", x, yAxisData);
         }   
			
        
         return rawPlot;
    }

	public PlotWindow extractSpectrum(ImageStack stack2, ImagePlus imp2, Roi roi, 
				double startEnergyDbl2, double stepEnergyDbl2, double endEnergyDbl2, double[] energies) {
         
	startEnergyDbl = startEnergyDbl2;
	stepEnergyDbl = stepEnergyDbl2;
	endEnergyDbl = endEnergyDbl2;        
	imp = imp2;
	stack = stack2;
 	
 			
        minThreshold = imp.getProcessor().getMinThreshold();
        maxThreshold = imp.getProcessor().getMaxThreshold();
        yAxisData = getZAxisProfile(roi, minThreshold, maxThreshold);

        float[] temp = new float[yAxisData.length]; 
        
        
        
        for (int i = 0; i<yAxisData.length; i++)
        {
        		temp[i] = yAxisData[i];
        
        }
        
        if(imp.getType() == 1)
        {
        		for (int i = 0; i<yAxisData.length; i++)
        		{
        			yAxisData[yAxisData.length-i-1] = temp[i]-32768;
        
        		}
			}
			else
			{
				for (int i = 0; i<yAxisData.length; i++)
        		{
        			yAxisData[yAxisData.length-i-1] = temp[i];
        
        		}
        	}
			
			double yAxisMax=0.0;
			double yAxisMin =40000;
			
			for (int i=0; i<yAxisData.length; i++)
			{
				if(yAxisData[i] > yAxisMax)
				{
					yAxisMax = yAxisData[i];
				}
				
				if(yAxisData[i] < yAxisMin)
				{	
					yAxisMin = yAxisData[i];
				}
				
			}
			
			
			
			
        
            
			if(yAxisData!=null)
			{
				float[] x = new float[yAxisData.length];
            for (int i=0; i<x.length; i++)
                x[i] = (float)(energies[stack.getSize()-1-i]);
            Rectangle r = imp.getRoi().getBoundingRect();
            rawPlot = new PlotWindow(imp.getTitle()+"-"+r.x+"-"+r.y, "Energy", "Mean", x, yAxisData);
				rawPlot.setLimits(energies[stack.getSize()-1], energies[0], yAxisMin, yAxisMax);
			}
			
        
         return rawPlot;
    }

   public float[] getZAxisProfile(Roi roi, double minThreshold, double maxThreshold) {
   
        int size = stack.getSize();
        float[] values = new float[size];
        ImageProcessor mask = imp.getMask();
        Rectangle r = imp.getRoi().getBoundingRect();
        Calibration cal = imp.getCalibration();
        Analyzer analyzer = new Analyzer(imp);
        int measurements = analyzer.getMeasurements();
        boolean showResults = measurements!=0 && measurements!=LIMIT;
        measurements |= MEAN;
        if (showResults) {
            if (!analyzer.resetCounter())
                return null;
        }
        for (int i=1; i<=size; i++) {
            ImageProcessor ip = stack.getProcessor(i);
            if (minThreshold!=ImageProcessor.NO_THRESHOLD)
                ip.setThreshold(minThreshold,maxThreshold,ImageProcessor.NO_LUT_UPDATE);
            ip.setRoi(r);
            ip.setMask(mask);
            ImageStatistics stats = ImageStatistics.getStatistics(ip, measurements, cal);
            analyzer.saveResults(stats, roi);
            //if (showResults)
                //analyzer.displayResults();
            
            if(imp.getType() == 0)
            {
            byte[] pixelTest = new byte[imp.getWidth() * imp.getHeight()];
            pixelTest = (byte[]) (stack.getPixels(1));
           
            int pixelTestFirst = (int) (pixelTest[0] & 0xffff);
            int pixelTestWidth = (int) (pixelTest[imp.getWidth()-1] & 0xffff);
            int pixelTestLast = (int) (pixelTest[imp.getWidth()*imp.getHeight() -1] & 0xffff);
            
         
            if((pixelTestFirst > 200 && pixelTestWidth > 200 && pixelTestLast > 200) && (imp.getType() == 0) ){
                
            	values[i-1] =255 - (float)stats.mean;
            
            }
            
            else {
            	 values[i-1] = (float)stats.mean;
            }
            
            }
            else {
            	 values[i-1] = (float)stats.mean;
            	 }
        }
        return values;
    }
        
        
   public float[] xAxisData(double startEnergyDbl2, double stepEnergyDbl2, 
   double endEnergyDbl2, ImageStack stack2)
   {
   	startEnergyDbl = startEnergyDbl2;
	   stepEnergyDbl = stepEnergyDbl2;
	   endEnergyDbl = endEnergyDbl2; 
	   stack = stack2;
	   
	    
   	float[] x = new float[stack.getSize()];
      for (int i=0; i<x.length; i++)
      x[i] = (float)(endEnergyDbl +(Math.abs(stepEnergyDbl) * i));
      return x;
   }
   
   public float[] xAxisData(double startEnergyDbl2, double stepEnergyDbl2, 
   double endEnergyDbl2, ImageStack stack2, double[] energies)
   {
   	startEnergyDbl = startEnergyDbl2;
	   stepEnergyDbl = stepEnergyDbl2;
	   endEnergyDbl = endEnergyDbl2; 
	   stack = stack2;
	   
	    
   	float[] x = new float[stack.getSize()];
      for (int i=0; i<x.length; i++)
      x[i] = (float)(energies[stack.getSize()-1-i]);
      return x;
   }
   
   public float[] yAxisData(double startEnergyDbl2, double stepEnergyDbl2, 
   double endEnergyDbl2, ImageStack stack2, ImagePlus imp2, Roi roi)
   {	
   	startEnergyDbl = startEnergyDbl2;
		stepEnergyDbl = stepEnergyDbl2;
		endEnergyDbl = endEnergyDbl2;        
		imp = imp2;
		stack = stack2;
   		
   	  minThreshold = imp.getProcessor().getMinThreshold();
        maxThreshold = imp.getProcessor().getMaxThreshold();
        yAxisData = getZAxisProfile(roi, minThreshold, maxThreshold);

        float[] temp = new float[yAxisData.length]; 
        
        
        
        for (int i = 0; i<yAxisData.length; i++)
        {
        		temp[i] = yAxisData[i];
        
        }
        
        
        for (int i = 0; i<yAxisData.length; i++)
        {
        		yAxisData[yAxisData.length-i-1] = temp[i];
        
        }
   
   		return yAxisData;
   
   
   }
   
   
     
       
/**************************
*  Takes the extracted spectrum and then normalizes the data.
***************************/

    public PlotWindow normalizeSpectrum() {
        float baseline = (float)999999;

        if (yAxisData!=null) {
            // First, normalize.
            yAxisNormData = yAxisData;
        
                for (int i = 0; i < yAxisNormData.length; i ++) {
                    if (yAxisNormData[i] < baseline)
                        baseline = yAxisNormData[i];
                }
                for (int i = 0; i < yAxisNormData.length; i ++) {
                    yAxisNormData[i] = yAxisNormData[i] - baseline;
                }
            
            float[] x = new float[yAxisNormData.length];
            for (int i=0; i<x.length; i++)
                x[i] = (float)(endEnergyDbl +(Math.abs(stepEnergyDbl) * i));
            Rectangle r = imp.getRoi().getBoundingRect();
            normPlot = new PlotWindow(imp.getTitle()+"-"+r.x+"-"+r.y, "Energy", "Mean", x, yAxisNormData);
           

           
        }
                   return normPlot;
    }
      
}