import ij.*;
import ij.gui.*;
import ij.measure.*;
import ij.plugin.*;
import ij.plugin.filter.*;
import ij.plugin.frame.*;
import ij.process.*;
import ij.util.*;
import java.awt.*;
import java.awt.event.*;
import java.lang.Math;
import java.lang.*;
import java.io.FileWriter;
import java.io.*;
import java.util.StringTokenizer;
import java.text.*;
import java.util.*;
import ij.process.*;
import ij.util.Java2;
import java.util.Stack;
import ij.text.TextWindow;
import ij.io.*;

/**
This class runs everything. It creates two windows, one with buttons that
extract the spectrum, manage the ROI's, create a divison map, extract the spectrum
from the ROI's that have been copied to the ROI window, and sets the preferences of the application.
The second window displays the starting, ending, step, on and off peak energies. Finally, it creates
a mouse listener that updates the on and off peak energies when a user clicks on the spectrum window   
*/

public class SpectrumExtractor_ extends PlugInFrame implements ActionListener, Measurements, MouseListener{
	
	 //Variables needed to make the program work like it should
    protected ImageStack stack;
    private ImageCanvas ic;
    private ImageCanvas divCan;
    private ImageCanvas normic;
    protected Roi roi;
    static protected ImagePlus imp;
    
    private float[] yAxisData;
    private float[] yAxisNormData;
    private ExtractSpectrum extractor = new ExtractSpectrum();
    private ROIManager roiManage = new ROIManager();

    private PlotWindow rawPlot;
    private PlotWindow normPlot;
    private ImageWindow distIw;
    private ImageWindow onIw;
    private ImageWindow offIw;
    private ImageWindow wayOffIw;
    private ImageWindow diffIw;
    private ImageWindow divIw;

	 private Panel buttonPanel = new Panel();
    private Panel mainPanel = new Panel();
    private Panel dataPanel =  new Panel();
    private Panel middlePanel = new Panel();
    private Panel peakPanel = new Panel();
    private Panel prefPanel = new Panel();
    private Panel invertMapPanel = new Panel();
	 private	Panel peaksPanel = new Panel();
    
    private BorderLayout cbl = new BorderLayout(4,4);

    private Label startEnergyLab = new Label("Start Energy:");
    private Label endEnergyLab = new Label("End Energy:");
    private Label stepEnergyLab = new Label("Energy Step:  -");
    private Label negative = new Label("-");
    private Label onPeakLabel = new Label("On Peak:");
    private Label offPeakLabel = new Label("Off Peak:");
    private Label invertDivMapLabel = new Label("  Invert Divison Map  :");
    private Label displayPeaksLabel = new Label("Display Peak Slices:");

    private String energyUnitsStr = "eV";
    private String spacerStr = " ";

	 private double startEnergyDbl = 0;
    private double stepEnergyDbl = 0;
    private double endEnergyDbl = 0;
    private double onPeakEnergyDbl = 0;
    private double offPeakEnergyDbl = 0;
    private int counter = 0;
    private int xcounter = 0;
    private int roiCounter = 1;
    private int[] pixels = new int[2];
    private double[] energies;
    private int onPeakSlice =0; 
    private int offPeakSlice=0;

    private TextField startEnergyTxt = new TextField("" + startEnergyDbl);
    private TextField endEnergyTxt = new TextField("" + endEnergyDbl);		
    private TextField stepEnergyTxt = new TextField("" + stepEnergyDbl);
    private TextField onPeakEnergyTxt = new TextField("" + onPeakEnergyDbl);
    private TextField offPeakEnergyTxt = new TextField("" + offPeakEnergyDbl);;

    private Checkbox displayPeaksCB = new Checkbox();
    private Checkbox invertDivMapCB = new Checkbox();
    
    private boolean converting = true;   
    private boolean readingvalues = true;
    
    private Choice normChoice = new Choice();

    static protected boolean reversed = true;
    private boolean datFile = false;
    private boolean showPeakImages = true;

    private Button extractButton = new Button("Gen Spectrum");
    private Button normalizeButton = new Button("Normalize");
    private Button divMapButton = new Button("Division Map");
    private Button rescaleButton = new Button("Rescale");
    private Button roiCopyButton = new Button("Copy Roi");
    private Button extractRoiButton = new Button("Roi Spectrum");
    private Button preferencesButton = new Button("Preferences");
    private Button resetEnergyButton = new Button("Reset Energies"); 
    private Button datEnergyButton = new Button("Energies from Dat");
    private NumberFormat formatter = NumberFormat.getNumberInstance();
    
    private Menu buttons;
                               
    
    //declaration needed for PlugInFrame to be extended
    public SpectrumExtractor_() {
        super("SpectrumExtractor");
    }
	
	 /**This controls what happens when the program first starts up.
	 */
    public void run(String arg) {
    
    	  if (arg.equals("about")) {
            showAbout(); return;
            }
            
        imp = WindowManager.getCurrentImage();
        
        
        //if there is no current image, tell the user to open one up.
        if (imp == null) {
            IJ.showMessage("SpectrumExtractor_",
            "Please load the stack and select a region of interest before\n" +
            "starting the Spectrum Extracter.");
            return;
            
        	}
        	 
        else {
            stack=imp.getStack();
            roi=imp.getRoi();
            
           //The first time the plugin runs this reads the values in for the energies in from 
           // a text file if they exist, and starts the dialog boxes that allow the user to change
           // what the starting values of the energies are. It also formats doubles to only one 
           // decimal place.   			
           if (readingvalues) 
           {
           	  formatter.setMaximumFractionDigits(1);
		        formatter.setMinimumFractionDigits(1);
				  startEnergy(); 
				  energyDialog();

	           readingvalues = false;
		 		}      	             
        }
        
        //sets the layout of the window with the energies
        setlayout();
        
        //If an Roi is selected when the program first starts up, then extract the spectrum from it
        // and let the user normalize the spectrum and take a divison map.
        if (imp.getRoi() != null) {
			  normalizeButton.setEnabled(true);
			  divMapButton.setEnabled(true);
			  extractSpectrum();
			}
        this.show();
        this.repaint();
        
        //If there is a region of interest put the spectrum window in front.
        if(imp.getRoi() != null){
        	  IJ.selectWindow(rawPlot.getTitle());
           }
        //run the brightness/contrast option of ImageJ   
        IJ.run("Brightness/Contrast..."); 
    }
	
	 /** This sets the layout of the window containing the energies of everything.
	 It also adds actionlisteners to all the buttons, and text areas. It also creates
	 a frame displaying the buttons that control what you can do with the program and sets
	 the location for that frame. Finally, it sets the ending energy to what it should be instead
	 of zero based on the number of slices in the stack, the starting energy and the step energy.
	 */
    private void setlayout() {
    
    	  startEnergyTxt.addActionListener(this);
    	  endEnergyTxt.addActionListener(this);
    	  stepEnergyTxt.addActionListener(this);
    	  onPeakEnergyTxt.addActionListener(this);
    	  offPeakEnergyTxt.addActionListener(this);
    	  
    	  preferencesButton.addActionListener(this);
    	  extractButton.addActionListener(this);
 		  normalizeButton.addActionListener(this);
		  divMapButton.addActionListener(this);
		  roiCopyButton.addActionListener(this); 
    	  extractRoiButton.addActionListener(this);
    	  resetEnergyButton.addActionListener(this);
    	  rescaleButton.addActionListener(this);
    	  datEnergyButton.addActionListener(this);
    	  
    	  
    	  buttons=new Menu();
    	  buttons.setLocation(670,90);
    	  
    	  endEnergyTxt.setEditable(false);
    	  startEnergyTxt.setEditable(false);
    	  stepEnergyTxt.setEditable(false);
    	  onPeakEnergyTxt.setEditable(false);
    	  offPeakEnergyTxt.setEditable(false);
    	  
    	  if(endEnergyDbl != startEnergyDbl +((double)(imp.getStackSize()-1)*(-stepEnergyDbl)) );
    	  {
        		endEnergyDbl = startEnergyDbl +((double)(imp.getStackSize()-1)*(-stepEnergyDbl));
        		endEnergyTxt.setText("" + endEnergyDbl);
        }
        
                
        this.setSize(600,100);
        this.setLayout(cbl);
        startEnergyTxt.setColumns(7);
        endEnergyTxt.setColumns(7);
        onPeakEnergyTxt.setColumns(7);
        offPeakEnergyTxt.setColumns(7);

        dataPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
        dataPanel.add(startEnergyLab);
        dataPanel.add(startEnergyTxt);
        dataPanel.add(new Label(energyUnitsStr));

        dataPanel.add(new Label(spacerStr));
        dataPanel.add(endEnergyLab);
        dataPanel.add(endEnergyTxt);
        dataPanel.add(new Label(energyUnitsStr));
        
        dataPanel.add(new Label(spacerStr));
        dataPanel.add(stepEnergyLab);
        dataPanel.add(stepEnergyTxt);
        dataPanel.add(new Label(energyUnitsStr));
        dataPanel.setBounds(0,0,100,100);
        dataPanel.setSize(100,100);
        this.add(dataPanel, BorderLayout.NORTH);

        peakPanel.setLayout(new FlowLayout(FlowLayout.CENTER));
        peakPanel.add(onPeakLabel);
        peakPanel.add(onPeakEnergyTxt);
        peakPanel.add(new Label(energyUnitsStr));

        peakPanel.add(new Label(spacerStr));
        peakPanel.add(offPeakLabel);
        peakPanel.add(offPeakEnergyTxt);
        peakPanel.add(new Label(energyUnitsStr));
        
        middlePanel.setLayout(new BorderLayout());
        middlePanel.add(peakPanel, BorderLayout.NORTH);
        this.add(middlePanel, BorderLayout.CENTER);

    }

	 /**A method that isn't used and probably will be deleted soon
	 */
    public void run(ImageProcessor ip) {
        IJ.showMessage("SpectrumExtracter_","Hello world!");
    }

	 /**A method that displays a message about what this program does
	 */
    void showAbout() {
        IJ.showMessage("About SpectrumExtractor_...",
        "This plugin is designed to allow you to extract\n" +
        "information from the grey levels of a stack of\n" +
        "images in a Region of Interest.");
    }
   
    /**This method calls another class and divides one image at the on peak energy 
    by another image at the off peak energy. It checks to see if the image is 8 or 16bit 
    and calls the apropriate class.
	 */
    public void divMap()
		{
			if(imp.getType() == 0)
			{
				DivisonMap8Bit divMap8bit = new DivisonMap8Bit();
         	divMap8bit.divisonMap(onPeakEnergyDbl, offPeakEnergyDbl, startEnergyDbl, 
         	stepEnergyDbl, endEnergyDbl, stack, imp, invertDivMapCB.getState(), onPeakSlice, offPeakSlice);
			
			
			}
			if(imp.getType() == 1){
				DivisonMap16Bit divMap16bit = new DivisonMap16Bit();
         	divMap16bit.divisonMap(onPeakEnergyDbl, offPeakEnergyDbl, startEnergyDbl, 
         	stepEnergyDbl, endEnergyDbl, stack, imp, invertDivMapCB.getState(), onPeakSlice, offPeakSlice);
         }

		}	
    
    //Stuff needed for the mouselistener class that isn't used
    public void mousePressed(MouseEvent e) { }
    public void mouseReleased(MouseEvent e) {}
    
    /**This method takes the pixel value where the mouse clicks in the spectrum window 
    and traslates them through the peakfinder class into the corresponding energy. It uses
    a counter to keep track of what whether the user is clicking on the on or off peaks.
    */
    public void mouseClicked(MouseEvent e) {
      	int pixel =0;
      
       if (xcounter == 0)
       {
          pixels[0]= e.getX();
          pixel=pixels[0];
          pixels[0] = peakfinder(pixel);
          IJ.makeLine(pixels[0], 21, pixels[0], 219);
          IJ.run("Draw");
          
          xcounter++;
       }
       else if(xcounter == 1)
       {
          pixels[1]= e.getX();
          pixel=pixels[1];
      	 pixels[1] = peakfinder(pixel);
      	 IJ.makeLine(pixels[1], 21, pixels[1], 219);
      	 IJ.run("Draw");
      	 
      	 xcounter++;
       }
       else
       {
          IJ.makeLine(pixels[0], 21, pixels[0], 219);
          IJ.run("Clear");
          IJ.makeLine(pixels[1], 21, pixels[1], 219);
          IJ.run("Clear");
          IJ.makeLine(0,0,0,0);
        
      	 xcounter=0;
       } 
      
      	}
      	
    //More stuff needed for the mouselistener class that isn't used  	
    public void mouseExited(MouseEvent e) {}
    public void mouseEntered(MouseEvent e) {}
	 
	 /**This method handles what happens when you hit all those pretty buttons I created.
	  and lets you know if you don't have a roi selected when you need one, if you are trying
	  to do spectral analysis on a single picture, and checks the data to make sure it makes sense.
	 */
    public void actionPerformed(ActionEvent evt) 
    {
    		
    		Object src = evt.getSource();
    
        if (imp == null) {
            IJ.showMessage("Spectrum Extractor","This command requires a stack be loaded.");
            return;
        }
        if (imp.getStackSize()<2) {
        		imp = WindowManager.getCurrentImage();
         	IJ.showMessage("Spectrum Extractor", "This command requires a stack.");
            return;
        }
        
        if(src == preferencesButton)
        
        {
        		Pref pref = new Pref();
        		pref.setLocation(670,200);
        }
        
        if(src == resetEnergyButton)
        {
        	  buttons.close();	
        	  this.close();
        	  IJ.runPlugIn("SpectrumExtractor_","");
        }
        
        if (src == datEnergyButton)
        {
        		datEnergy();
        }
        
        if ((imp.getRoi() == null) && ( (src == extractButton) || (src == normalizeButton) ||
         	(src == roiCopyButton)) ) 
         	{
            	IJ.showMessage("Spectrum Extractor", "Please select a Region of Interest.");
            	return;
        }
        
        else {
        		Roi roi = imp.getRoi();
        
        		if (roi.getType()>=Roi.LINE) {
            	IJ.showMessage("Spectrum Extractor", "This command does not work with line selections.");
            	return;
        			}
        }
        if (validateData() == false){
            IJ.showMessage("Spectrum Extractor_ Error",
            "You must enter valid numbers for the Energy levels.");
            return;
        } else {
            
        }

        if (src == extractButton)
        {
            extractSpectrum();
            normalizeButton.setEnabled(true);
            divMapButton.setEnabled(true);
        }
        if (src == normalizeButton)
        {
            normSpectrum();
        }
		  
		  if(src == rescaleButton)
		  {
		  		rescale();
		  		rescaleButton.setEnabled(false);
		  }
        if(src == divMapButton)
        {
            divMap();
        }
        
       if (src == roiCopyButton)
        
        {
        		roiManager();
        		extractRoiButton.setEnabled(true);
        }
        
       if (src == extractRoiButton)
        {
     			extractAllSpectrum();
     			extractRoiButton.setEnabled(false);
        }
    
        this.repaint();
        	
    }

	 /**This method extracts the spectrum from a roi and returns a plot based on it. The
	 plot is then displayed and a mouselistener is added to it.
	 */
    public void extractSpectrum() {
       if(datFile==false)
       {
		 	 rawPlot = extractor.extractSpectrum(stack, imp, roi, startEnergyDbl, stepEnergyDbl, endEnergyDbl);
       }
       else
       {
       	 rawPlot = extractor.extractSpectrum(stack, imp, roi, startEnergyDbl, stepEnergyDbl, endEnergyDbl, energies);
       }
       
       rawPlot.draw();
       ic = rawPlot.getCanvas();
       ic.addMouseListener(this);
    	 } 
        
       

	 /**This method normalizes the spectrum from a roi and returns a plot based on it. The
	 plot is then displayed and a mouselistener is added to it.
	 */
    public void normSpectrum() {
       
		 normPlot = extractor.normalizeSpectrum();
       normPlot.draw();
       normic = normPlot.getCanvas();
       normic.addMouseListener(this);
       
    }
    
    
    public void rescale() {
    			roi = imp.getRoi();
    	 		short[] pixelTest = (short[])(stack.getPixels(1));
            int pixelTestFirst = (int) (pixelTest[0] & 0xffff);
            int pixelTestWidth = (int) (pixelTest[imp.getWidth()-1] & 0xffff);
            int pixelTestLast = (int) (pixelTest[imp.getWidth()*imp.getHeight() -1] & 0xffff);
            
            
            	if (converting == true && pixelTestFirst > 30000 
            		&& pixelTestWidth > 30000 && pixelTestLast > 30000) {

            		 	IJ.run("Select All");

	    					for (int i=1; i <= stack.getSize(); i++) 
	    					{

	    						IJ.run("AND...", "value=111111111111");
	    						IJ.run("Next Slice [>]");
               		}
               		
               		IJ.run("Set Slice...", "slice=1");
               		IJ.resetMinAndMax();
               		imp.setRoi(roi);
               		converting = false;
             	  }
    
    
    
    
     }
    
    /**This method adds roi's to a new window, numbers them and adds them to a stack
	 */
    public void roiManager()
  		{
  			roiManage.roiManager(imp, startEnergyDbl, stepEnergyDbl);
  		}
	
	/**This method extracts a spectrum from all the roi's that were created using 
	the roiManager, and displays a textWindow listing the different roi's, their graylevels
	and their energies 
	*/
	 public void extractAllSpectrum() 
	 {
	 	 if(datFile == false)
	 	 {
		 	 roiManage.extractAllSpectrum(endEnergyDbl, stack); 
		 }
		 
		 else
		 {
		 	 roiManage.extractAllSpectrum(endEnergyDbl, stack, energies);
		 }
	 }
	 
	 /**Checks to see if there are actual numbers corresponding to all the doubles 
	 we need to run the program.
	 */
    public boolean validateData(){

        try {
            startEnergyDbl = new Double(startEnergyTxt.getText()).doubleValue();
            endEnergyDbl = new Double(endEnergyTxt.getText()).doubleValue();
            stepEnergyDbl = new Double(stepEnergyTxt.getText()).doubleValue();
            onPeakEnergyDbl = new Double(onPeakEnergyTxt.getText()).doubleValue();
            offPeakEnergyDbl = new Double(offPeakEnergyTxt.getText()).doubleValue();

        }
        catch (Exception e) {
            return false;
        }
        return true;
    }
    
    /**Creates a frame displaying all the buttons used in the program.
	 */
    public class Menu extends PlugInFrame{
		 public Menu()
		 {
			 super("Spectrum_Extractor");
			 
			 setLayout(new FlowLayout());
			 buttonPanel.setLayout(new GridLayout(8, 1));
			
			 extractButton.setEnabled(true);
			 buttonPanel.add(extractButton);
			 divMapButton.setEnabled(false);
          buttonPanel.add(divMapButton);
          
          if(imp.getType() == 1)
          {
          	 rescaleButton.setEnabled(true);

          }
          else
          {
          	 rescaleButton.setEnabled(false);
          } 
          //buttonPanel.add(rescaleButton);
          normalizeButton.setEnabled(false);
          buttonPanel.add(normalizeButton);
          roiCopyButton.setEnabled(true);
          buttonPanel.add(roiCopyButton);
          extractRoiButton.setEnabled(false);
          buttonPanel.add(extractRoiButton);
          resetEnergyButton.setEnabled(true);
          buttonPanel.add(resetEnergyButton);
          datEnergyButton.setEnabled(true);
          buttonPanel.add(datEnergyButton);
          preferencesButton.setEnabled(true);
          buttonPanel.add(preferencesButton);
			 add(buttonPanel);
		
			 pack();
			 show();
	
		 }
	}
	 
	 /**Creates a Frame displaying all the preferences for SpectrumExtractor
	 */
	 public class Pref extends PlugInFrame{
		 public Pref()
		 {
			 super("Preferences");
			 
			 setLayout(new GridLayout(1,1));
			
			 prefPanel.setLayout(new GridLayout(2,1));
			
			 invertMapPanel.add(invertDivMapLabel);
			 invertDivMapCB.setState(true);
          invertMapPanel.add(invertDivMapCB);
          prefPanel.add(invertMapPanel);
         
			 peaksPanel.add(displayPeaksLabel);
          peaksPanel.add(displayPeaksCB);
          prefPanel.add(peaksPanel);
         
			 add(prefPanel);
    
			 pack();
			 show();
	
		 }
	}
	 
	 /**This method takes a pixel value and finds the corresponding energy. It uses a counter
	 so that the first time it is run it finds the onPeak energy, and the second it finds the
	 offPeak energy caclulate the slice location from the energy.  Remember, the stacks
    are set up backwards in energy.  So calculate where it would be in forwards
    order and subtract that from the size of the stack.
    */
    
	 public int peakfinder(int pixel){
	 	 int roundedPixel = 0;
	 	 formatter.setMaximumFractionDigits(1);
	    formatter.setMinimumFractionDigits(1);
	    
	 	 
	 	 if(datFile == false)
	 	 {
	 	 
	    try
	    {
	    
    	 if(counter == 0) {

    	 	 onPeakEnergyDbl =(endEnergyDbl + (pixel - 50.0)*((startEnergyDbl - endEnergyDbl) / 450.0 ) );
    	 	 
    	 	 onPeakSlice = stack.getSize() - (int)Math.round((onPeakEnergyDbl - endEnergyDbl)/Math.abs(stepEnergyDbl));
        
		    onPeakEnergyDbl = ( (stack.getSize() - onPeakSlice)*Math.abs(stepEnergyDbl) + endEnergyDbl);
		  
    		 String onPeakEnergyString = formatter.format(onPeakEnergyDbl);
    		 onPeakEnergyDbl = formatter.parse(onPeakEnergyString).doubleValue();	
    		 roundedPixel = (int) (50.0 + (onPeakEnergyDbl - endEnergyDbl) /((startEnergyDbl - endEnergyDbl) / 450.0 ));
    		 onPeakEnergyTxt.setText("" + onPeakEnergyDbl);
   		 counter++;	 
    		}
    
   	 else if(counter == 1) {
   	 
   	
    		 offPeakEnergyDbl = (endEnergyDbl + (pixel - 50.0)*((startEnergyDbl - endEnergyDbl) / 450.0 ));
    		 
    		 offPeakSlice = stack.getSize() - (int)Math.round((offPeakEnergyDbl - endEnergyDbl)/Math.abs(stepEnergyDbl));
    		 
    		 offPeakEnergyDbl = ( (stack.getSize() - offPeakSlice)*Math.abs(stepEnergyDbl) + endEnergyDbl);
    		 
    		 String offPeakEnergyString = formatter.format(offPeakEnergyDbl);
    		 offPeakEnergyDbl = formatter.parse(offPeakEnergyString).doubleValue();
    		 roundedPixel = (int) (50.0 + (offPeakEnergyDbl - endEnergyDbl) /((startEnergyDbl - endEnergyDbl) / 450.0 )); 
    		 offPeakEnergyTxt.setText("" + offPeakEnergyDbl);
    		 counter--;
    		}
    		
    	  }
    	  
    	  
    	  catch (ParseException e){}
    	  }
    	  
    	  else
    	  {
    	  		startEnergyDbl = energies[0];
		 		endEnergyDbl = energies[stack.getSize()-1];
    	  		try
	    {
	    
    	 if(counter == 0) {
				
			 onPeakSlice =0;	
    	 	 onPeakEnergyDbl =(endEnergyDbl + (pixel - 50.0)*((startEnergyDbl - endEnergyDbl) / 450.0 ) );
    	 	 String onPeakEnergyString = formatter.format(onPeakEnergyDbl);
    		 onPeakEnergyDbl = formatter.parse(onPeakEnergyString).doubleValue();
    		 //IJ.showMessage("endEnergy: " + endEnergyDbl + "On Peak: " + onPeakEnergyDbl);
    	 	 
    	 	 
    	 	 
    	 	 for(int i=0; i<stack.getSize(); i++)
    	 	 {
    	 	 	 if(onPeakEnergyDbl == energies[i])
    	 	 	 {
    	 	 	 	 onPeakSlice = i+1;	
    	 	 	 }
    	 	 }
    	 	 
    	 	 
    	 	 for(int j=0; j<100; j++)
    	 	 {
    	 	 	if(onPeakSlice == 0)
    	 	 	{
    	 	 	 	onPeakEnergyDbl=onPeakEnergyDbl-0.1;
    	 	 	 	onPeakEnergyString = formatter.format(onPeakEnergyDbl);
    		 		onPeakEnergyDbl = formatter.parse(onPeakEnergyString).doubleValue();
    	 	 	 	//IJ.showMessage(""+ onPeakEnergyDbl);
    	 	 	 
    	 	 	 	for(int i=0; i<stack.getSize(); i++)
    	 	 		{
    	 	 	 		if(onPeakEnergyDbl == energies[i])
    	 	 	 		{
    	 	 	 	 		onPeakSlice = i+1;	
    	 	 	 		}
    	 	 		}
    	 	 
    	 	 	}
    	 	 }	
    	 		//IJ.showMessage(""+onPeakSlice);
    		 roundedPixel = (int) (50.0 + (onPeakEnergyDbl - endEnergyDbl) /((startEnergyDbl - endEnergyDbl) / 450.0 ));
    		 onPeakEnergyTxt.setText("" + onPeakEnergyDbl);
   		 counter++;	 
    		}
    
   	 else if(counter == 1) {
   	 
   	 	 offPeakSlice =0;
    		 offPeakEnergyDbl = (endEnergyDbl + (pixel - 50.0)*((startEnergyDbl - endEnergyDbl) / 450.0 ));
    		 
    		 String offPeakEnergyString = formatter.format(offPeakEnergyDbl);
    		 offPeakEnergyDbl = formatter.parse(offPeakEnergyString).doubleValue();
    	 	 
    	 	 
    	 	 
    	 	 for(int i=0; i<stack.getSize(); i++)
    	 	 {
    	 	 	 if(offPeakEnergyDbl == energies[i])
    	 	 	 {
    	 	 	 	 offPeakSlice = i+1;	
    	 	 	 }
    	 	 
    	 	 }
    	 	 
    	 	 
    	 	 for(int j=0; j<100; j++)
    	 	 {
    	 	 	if(offPeakSlice == 0)
    	 	 	{
    	 	 	 	offPeakEnergyDbl=offPeakEnergyDbl-0.1;
    	 	 	 	offPeakEnergyString = formatter.format(offPeakEnergyDbl);
    		 		offPeakEnergyDbl = formatter.parse(offPeakEnergyString).doubleValue();
    	 	 	 	
    	 	 	 
    	 	 	 	for(int i=0; i<stack.getSize(); i++)
    	 	 		{
    	 	 	 		if(offPeakEnergyDbl == energies[i])
    	 	 	 		{
    	 	 	 	 		offPeakSlice = i+1;	
    	 	 	 		}
    	 	 		}
    	 	 
    	 	 	}
    	 	 }
    		 //IJ.showMessage(""+offPeakSlice);
    		 roundedPixel = (int) (50.0 + (offPeakEnergyDbl - endEnergyDbl) /((startEnergyDbl - endEnergyDbl) / 450.0 )); 
    		 offPeakEnergyTxt.setText("" + offPeakEnergyDbl);
    		 counter--;
    		}
    		
    	  }
    	  
    	  
    	  catch (ParseException e){}
    	  
    	  
    	  }
    	  
    	  return roundedPixel;
  		}  
   
   /**This Dialog box is run at the beginning of the program and sets what
   the starting and step energies will be.
	*/ 
   public void energyDialog()
   {

  		GenericDialog gd1 = new GenericDialog("Starting Energy:");
		gd1.addNumericField("Starting Energy", startEnergyDbl,1);
		gd1.showDialog();
		if (gd1.wasCanceled()) {
		   IJ.error("Setting Energies canceled!");
		   return;
		}
		
		startEnergyDbl = (double) gd1.getNextNumber();
		
		GenericDialog gd2 = new GenericDialog("Step Size:");
		gd2.addNumericField("Step Size is: -", stepEnergyDbl,1);
		gd2.showDialog();
		if (gd2.wasCanceled()) {
		   IJ.error("Setting Energies canceled");
		   return;
		}
		
		stepEnergyDbl = (double) gd2.getNextNumber();
				
		stepEnergyTxt.setText("" + stepEnergyDbl);
		startEnergyTxt.setText("" + startEnergyDbl);
		
		if(endEnergyDbl != startEnergyDbl +((double)(imp.getStackSize()-1)*(-stepEnergyDbl)) );
    	  	  {
        		  endEnergyDbl = startEnergyDbl +((double)(imp.getStackSize()-1)*(-stepEnergyDbl));
        		  endEnergyTxt.setText("" + endEnergyDbl);
           }
           
       this.repaint();
	}
		    
	 /**This method checks to see if there is a file with energies in it and then sets the
	 energies to those in the file.
	 */
    public void startEnergy() 
    {
       startEnergy startingEnergies = new startEnergy();
       double[] energyvalues = startingEnergies.startEnergy();

		 stepEnergyDbl = energyvalues[3];
		 startEnergyDbl = energyvalues[2];
		 onPeakEnergyDbl = energyvalues[0];
		 offPeakEnergyDbl = energyvalues[1];
	
		 stepEnergyTxt.setText("" + stepEnergyDbl);	
		 onPeakEnergyTxt.setText("" + onPeakEnergyDbl);
		 offPeakEnergyTxt.setText("" + offPeakEnergyDbl);
		 startEnergyTxt.setText("" + startEnergyDbl);

  	 }		
  	 
  	 public void datEnergy()
  	 
  	 {	
  	 	 datFile = true;
		 OpenDialog od = new OpenDialog("Dat File Opener", null);
       String name = od.getFileName();
       if (name==null)
       return;
       String dir = od.getDirectory(); 
       int numOfSlices = stack.getSize();
		 datEnergy datEnergies = new datEnergy();
		 energies=datEnergies.datEnergy(dir,name,numOfSlices);
		 startEnergyDbl = energies[0];
		 endEnergyDbl = energies[numOfSlices-1];
		 endEnergyTxt.setText("" + endEnergyDbl);
		 startEnergyTxt.setText("" + startEnergyDbl);
		 IJ.showMessage("Dat File Loaded!");
		 
  	} 
   
}
